from __future__ import annotations

from random import shuffle
from analysis.analysis import Analysis

from lang.cfg import CFG


def fixed_point(program: CFG, analysis: Analysis):    
    changed = True
    in_vals = {node: analysis.initial_in(node) for node in program.nodes}
    out_vals = {node: analysis.initial_out(node) for node in program.nodes}
    nodes = list(program.nodes)

    while changed:
        changed = False
        for node in nodes:
            predecessors_out = [out_vals[n] for n in program.previous[node]]
            successors_in = [in_vals[n] for n in program.next[node]]
            new_entry, new_exit = analysis.get_new_values(
                node, predecessors_out, successors_in)

            if (new_entry != in_vals[node]) or (new_exit != out_vals[node]):
                changed = True
            in_vals[node] = new_entry
            out_vals[node] = new_exit
    return (in_vals, out_vals)
