from __future__ import annotations
from typing import Set, Dict
from lang.ast import Statement, Variable


class Node:
    # Abstract Class

    def __init__(self) -> None:
        raise NotImplementedError

    def __lt__(self, o):
        return repr(o) < repr(self)


class InnerNode(Node):
    statement: Statement

    def __init__(self, statement) -> None:
        self.statement = statement

    def __str__(self) -> str:
        return f"{self.statement}"

    def __repr__(self) -> str:
        return f"<Node: {self.statement}>"


class Entry(Node):
    def __init__(self) -> None:
        pass

    def __str__(self) -> str:
        return "Entry"

    def __repr__(self) -> str:
        return "<Entry>"


class Exit(Node):
    def __init__(self) -> None:
        pass

    def __str__(self) -> str:
        return "Exit"

    def __repr__(self) -> str:
        return "<Exit>"


class CFG:
    entry: Entry
    exit: Exit
    nodes: Set[Node]
    next: Dict[Node, Set[Node]]
    previous: Dict[Node, Set[Node]]
    variables: Set[Variable]

    def __init__(self, variables: Set[Variable]) -> None:
        self.entry = Entry()
        self.exit = Exit()
        self.nodes = {self.entry, self.exit}
        self.next = {self.entry: set(), self.exit: set()}
        self.previous = {self.entry: set(), self.exit: set()}
        self.variables = variables

    def add_node(self, node: Node):
        if node not in self.nodes:
            self.previous[node] = set()
            self.next[node] = set()
            self.nodes.add(node)

    def str_next(self) -> str:
        sorted_nodes = sorted(list(self.nodes))
        return "\n".join(
            [f"{pad(str(k),20)} -> {list(map(str,sorted(list(self.next[k]))))}" for k in sorted_nodes])

    def str_prev(self) -> str:
        sorted_nodes = sorted(list(self.nodes))
        return "\n".join(
            [f"{pad(str(k),20)} -> {list(map(str,sorted(list(self.previous[k]))))}" for k in sorted_nodes])

    def __str__(self) -> str:
        return f"CFG: \n\nnext: \n{self.str_next()}\n\nprevious: \n{self.str_prev()}"

def pad(s: str, n: int) -> None:
    if len(s) > n:
        return s
    return s + (" " * (n - len(s)))
