from __future__ import annotations

from typing import Any, Optional, Set, Dict
from lang.ast import Statement, Expression, BinaryExpression, BooleanConstant, IntegerConstant, UnaryExpression, Assignment, Guard, Variable
from lang.cfg import Entry, Exit, InnerNode, Node

def getStatementUsedVars(stmt: Statement) -> Set[Variable]:
    if isinstance(stmt, Guard):
        return getExpressionUsedVars(stmt.expression)
    elif isinstance(stmt, Assignment):
        return getExpressionUsedVars(stmt.value)
    else:
        raise Exception(
            f"Statement {stmt} not supported for function getStatementUsedVars.")


def getExpressionUsedVars(expr: Expression) -> Set[Variable]:
    if isinstance(expr, Variable):
        return {expr}
    elif isinstance(expr, BinaryExpression):
        return getExpressionUsedVars(expr.lhs).union(getExpressionUsedVars(expr.rhs))
    elif isinstance(expr, UnaryExpression):
        return getExpressionUsedVars(expr.expression)
    elif isinstance(expr, (IntegerConstant, BooleanConstant)):
        return set()
    else:
        raise Exception(
            f"Expression {expr} not supported for function getExpressionUsedVars.")

def getValueByLabel(label: str, d: Dict[Node, Any]) -> Optional[Any]:
    for k in d.keys(): 
        if isinstance(k, InnerNode) and k.statement.label == label: 
            return d[k]
    return None

def getEntryValue(d: Dict[Node, Any]) -> Optional[Any]: 
    for k in d.keys():
        if isinstance(k, Entry):
            return d[k]
    return None

def getExitValue(d: Dict[Node, Any]) -> Optional[Any]: 
    for k in d.keys():
        if isinstance(k, Exit):
            return d[k]
    return None